function P = power(A,n);
% P=POWER(A,n) or P=A.^n
% compute the n-th element-wise power of the 2-D polynomial matrix
% A - POL2-class 2-D polynomial matrix
% n - nonnegative integer exponent, may be matrix compatible with A

if nargin<2,
   error('Not enough input arguments.');
end;
if ~isa(n,'double') | ndims(n)>2 | ~isreal(n) | ...
      (~isempty(n) & (any(any(n<0)) | any(any(floor(n)~=n)))),
   error('Invalid power; must be nonnegative integer.');
end;
nz = nonzeros(n);
if any(log2(nz)>=52),
   error('The power is too large. Use an argument up to 2^52.')
end;

ns = size(n);
if all(ns==1),           % n  is scalar
   P = scpower(A,n);
else                     % n  is matrix
   As = A.s;
   if all(As==1),                  % A  is scalar
      A = ones(ns)*A; As = A.s;
   else                            % A  is matrix
      if any(As~=ns),
         error('Matrices not of the same dimensions.');
      end;
   end;
   Ad = A.d;
   if isempty(Ad) | any(Ad<0),
      P = A;
   else
      nmax = max(max(n));
      Pc = zeros(As(1),As(2),nmax*Ad(1)+1,nmax*Ad(2)+1);
      Ac = A.c;
      for i = 1:As(1),
         for j = 1:As(2),
            Acij = Ac(i,j,:,:);
            Aij = pol2(Acij);
            Pij = scpower(Aij,n(i,j));
            [Ps1,Ps2,Ps3,Ps4] = size(Pij);
            if Ps3>=0,
               Pc(i,j,1:Ps3+1,1:Ps4+1) = Pij.c(1,1,1:Ps3+1,1:Ps4+1);            
            end;
         end;
      end;
      P = pol2(Pc);
   end;
end;

% Sub-function for scalar power
function P = scpower(A,n)

N = fliplr(num2str(dec2bin(n))); % power of 2 chain

Z = A;
F = pol2(ones(size(A)));
lN = length(N);
for k = 1:lN,
   if N(k)=='1',
      F = times(F,Z);
   end;   %if
   if k<lN,
      Z = times(Z,Z);		% no zeroing yet
   end;
end; 	%for
P = F;

%end .. scpower

%end .. pol2/power